--[[
Project name: DkpBidder
Developed by: Eadel
Website: http://www.arthion.fr/aldea
Description: DKP ingame management
License: All Rights Reserved unless otherwise explicitly stated. 
File: DKPBidder.lua
File description: Core
]]

--The list of bidders to be
local bidders = {};
--Will disable whispers and cheks for +1 while in combat to reduce load
local DKPBidder_incombat = false;
--Prefix to recognize whisps
local PREFIX = "DkpBidder : "

--Loading addon core
DKPBidder= LibStub("AceAddon-3.0"):NewAddon("DKPBidder","AceHook-3.0","AceConsole-3.0","AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("DKPBidder", true)

--Default Database
local defaultsDB = {
	global = {
		Hidewhisps = false,
		DkpInfo = {timestamp = "0",}

	}
}

--Options table
local options = {
    name = "DKPBidder",
    handler = DKPBidder,
    type = 'group',
    args = {
				--toggle addon on and off
				toggle = {
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable DKPBidder"],
					get = function(info)
							return DKPBidder:IsEnabled()
						end,
					set = function(info, value)
							--module.db.profile.enabled = not not value
							if value then
								return DKPBidder:Enable()
							else
								return DKPBidder:Disable()
							end
						end,
					order = 1
				},
				
				-- toggle whisper filter on and off
				Whisper = {
					type = 'toggle',
					name = L["Show whispers"],
					desc = L["Show/Hide whispers"],
					order = 2,
					get = function() 
						--print(DKPBidder.db.global.setting)
						--DKPBidder.db.global.setting=DKPBidder.db.global.setting+1
						--print(DKPBidder.db.global.setting)
						if DKPBidder.db.global.Hidewhisps then
							print (L["Whisps filter is off"])
						else
							print (L["Whisps filter is on"])
						end
						return DKPBidder.db.global.Hidewhisps end,
					set = function(info, value)
						DKPBidder.db.global.Hidewhisps = not DKPBidder.db.global.Hidewhisps
					end,
				},
			},
}
--registering config
LibStub("AceConfig-3.0"):RegisterOptionsTable("DKPBidder", options, {"dkpbidder"})

--Setting popup dialogs
--Called when +1 in raid
	StaticPopupDialogs["DKPBIDDER_PRINT_LIST"] = {
  		text = L["Print list"],
  		button1 = L["Yes"], 
  		button2 = L["No"], 
  		OnAccept				=	function(self)
										DKPBidder:printBidders()
									end,
		OnCancel				=	function(self)
									end,
  		timeout = 0,
  		whileDead = 1,
  		hideOnEscape = 1
	};
	
--Called when a loot is linked in raid channel
		StaticPopupDialogs["DKPBIDDER_CLEAR_LIST"] = {
  		text = L["Clear list"],
  		button1 = L["Yes"], 
  		button2 = L["No"], 
  		OnAccept				=	function(self)
										DKPBidder:wipeBidders()
									end,
		OnCancel				=	function(self)
										
									end,
  		timeout = 0,
  		whileDead = 1,
  		hideOnEscape = 1 -- quitter
	};

--Loading addon
function DKPBidder:OnInitialize()	
DKPBidder:RegisterChatCommand("dkp", "SlashCommand")
self.db = LibStub("AceDB-3.0"):New("DKPBidderDB",defaultsDB)
end


--Enabling addon
function DKPBidder:OnEnable()	


--  events
		DKPBidder:RegisterEvent("CHAT_MSG_WHISPER");
		DKPBidder:RegisterEvent("CHAT_MSG_RAID","eventHandler");
		DKPBidder:RegisterEvent("CHAT_MSG_RAID_LEADER","eventHandler");
		DKPBidder:RegisterEvent("PLAYER_REGEN_DISABLED");
		DKPBidder:RegisterEvent("PLAYER_REGEN_ENABLED");
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", BlockIncomingChatFrame)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", BlockOutgoingChatFrame)
		
		
		--Headcount check
		if (HeadCount) then
			DKPBidder:Hook(HeadCount,"lootManagementUpdate", "HeadcountHook", true)
		end
		
		
		print(L["addon.init"])
		
		--Check if DKP_list.lua is feeding addon
		if DKPInfo then -- file is present
			--checking internal database and dkp_list
			if (tonumber(DKPInfo.timestamp) > tonumber(self.db.global.DkpInfo.timestamp))  then 
				--dkp_list is more recent, erasing internal database
				print(L["DB.updating"])
				self.db.global.DkpInfo = DKPInfo
				self.db.global.DkpData = gdkp
			else 
				-- internal database is ok
				print (L["DB.uptodate"])
			end
			-- clearing memory of useless ressources
			--wipe(DKPInfo)
			--wipe(gdkp)
			wipe(DKP_ITEMS)
			wipe(GetDKPRaidPlaner)
		else
			print (L["DKPlist missing"])
			DKPBidder:Disable()
		end
end


-- Disabling addon
function DKPBidder:OnDisable()	
	-- Unhooking
	if DKPBidder:IsHooked(HeadCount,"lootManagementUpdate") then
		DKPBidder:Unhook(HeadCount,"lootManagementUpdate")
	end
	print (L["Addon Disabled"])
end

--------------------------
--Slashcommand handler /dkp
---------------------------
function DKPBidder:SlashCommand(input)

	--getting the differents args
	_args = GetArgs(input, " ");

	--print
	if (input == "print") then
			DKPBidder:printBidders();
			
	--clear
	elseif (input == "clear") then
		DKPBidder:wipeBidders()
		
	--add
	elseif (_args[1] == "add") then
		-- no second and third arg
		if (_args[2]==nil or _args[3]==nil) then
			print(L["AddManual"])
		else
			--is player in roster ?
			if DKPBidder:playerExists(_args[2]) then
				-- is the number valid ?
				if (tonumber(_args[3])==nil) then
					print(L["NotANumber"])
				else
					DKPBidder:DKP_Add(_args[2].." ".._args[3])
				end
			else
				print(_args[2]..L["notinroster"])
			end
		end
		
	--minus
	elseif (_args[1] == "minus") then
		-- no second and third arg
		if (_args[2]==nil or _args[3]==nil) then
			print(L["SubManual"])
		else
			if DKPBidder:playerExists(_args[2]) then
				if (tonumber(_args[3])==nil) then
					print(L["NotANumber"])
				else
					DKPBidder:DKP_Sub(_args[2].." ".._args[3])
				end
			else
				print(_args[2]..L["notinroster"])
			end
		end
	  
	else
		print (L["Addon.usage"]) 
		print (L["addon.usage2"]) 
		print (L["addon.usage3"]) 
		print (L["addon.usage4"]) 
	end
end


----------------
-- Event Handler
----------------
function DKPBidder:eventHandler(event)
	
	
	
	if (event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") then
		
		_args = GetArgs(arg1, " ");	
		
		-- Detect if an item has been linked in raid channel
		if DKPBidder_incombat == false then
				-- Grab item info if they exist
				local sName, sLink, iRarity = GetItemInfo(arg1)
				if (iRarity==4) then
					-- clear the list before the +1 !! quickly !!
					StaticPopup_Show("DKPBIDDER_CLEAR_LIST")
				end
		end
		
		-- Detect +1 in raid or raid leader chan, add bidder un bidder list
		if ((_args[1] == "+1")) then
				-- counter to count if a player does several +1 for one loot
				local count =0
				-- insert anyone (case table is empty)
				table.insert(bidders, arg2);
				
				-- loop the bidders
				for index,value in ipairs(bidders) do 
					-- the +1'er is already in the list, but it can be his first one
					if (string.find(value,arg2)~=nil) then
						count=count+1
						-- and here is the second +1, let's remove it
						if count==2 then
							table.remove(bidders)
							-- and reset the counter, is it useful ?
							count=0
						end
					end
				end;

				-- Propose to print the list after each +1, each popup overwrite the other cleanly, so when bid is over, just click yes
				StaticPopup_Show("DKPBIDDER_PRINT_LIST")
		end;
  	end;
end


------------------
-- Whisper handler
------------------
function  DKPBidder:CHAT_MSG_WHISPER()
		
		_args = GetArgs(arg1, " ");

		-- a member is requesting dkp
		if (_args[1] == "dkp") then
			-- for himself
			if (_args[2] ~= nil) then
				-- he asks for someone else
				if(DKPBidder:playerExists(_args[2])) then
					SendChatMessage(PREFIX..L["points.of"].._args[2].." : "..DKPBidder:grabDkp(_args[2]).." dkp.", "WHISPER", this.language, arg2);
				else 
				-- not in roster
					SendChatMessage(PREFIX.._args[2]..L["notinroster"],  "WHISPER", this.language, arg2);
				end
			-- and he is in the roster
			elseif(DKPBidder:playerExists(arg2)) then
				SendChatMessage(PREFIX..L["your.points"]..DKPBidder:grabDkp(arg2).." dkp.", "WHISPER", this.language, arg2);
			else 
			-- not in roster
				SendChatMessage(PREFIX..L["younotinroster"],  "WHISPER", this.language, arg2);
				
			
			end;
		end;
		
		-- old command to clear the bidder's list, to be deleted
		if (_args[1] == "clear") then
			DKPBidder:wipeBidders()
		end;
		-- old command to print the bidder's list, to be deleted
		if (_args[1] == "print") then

			DKPBidder:printBidders();

		end; -- end of stop cond
		
		
end; -- end of whisper event

function DKPBidder:PLAYER_REGEN_DISABLED()
	DKPBidder_incombat = true
end

function DKPBidder:PLAYER_REGEN_ENABLED()
	DKPBidder_incombat = false
end


--GetAgrs function, very useful, created with the help of Corgan
function GetArgs(message, separator)
	if (message == nil) then
		return;
	end;
	-- Declare 'args' variable.
	local args = {};

	-- Declare 'i' integer.
	i = 0;

	-- Search for seperators in the string and return
	-- the separated data.
	
	for value in string.gmatch(message, "[^"..separator.."]+") do
		i = i + 1;
		args[i] = value;
	end -- end for

	-- Submit the filtered data.
	return args;
end -- end GetArgs()

-- Print the bidders list
function DKPBidder:printBidders()
			local winner
			local winnerpoints=0
			-- simple loop to go through all bidders
			for index,value in ipairs(bidders) do 
				--is in roster
				if(DKPBidder:playerExists(value)) then
					SendChatMessage(index..L["points.of"]..value.." : "..DKPBidder:grabDkp(value).." dkp.", "RAID", this.language);
					--determining the winner
					if DKPBidder:grabDkp(value) > winnerpoints then
						winner = tostring(value)
						winnerpoints=DKPBidder:grabDkp(value)
					end
				--not in roster
				else
					SendChatMessage(index..". "..value..L["notinroster"], "RAID", this.language);
				end
			end;
			-- printing the winner
			SendChatMessage(L["The winner is"]..tostring(winner).." !", "RAID", this.language);
			DKPBidder:wipeBidders()
end


--wipe the bidders, looking for a more optimized way to do it
function DKPBidder:wipeBidders()
	wipe(bidders);
	print(L["BiddersCleared"])
end

-- --------------------
--       MATH FUNCTIONS
-- -------------------

-- Minus
function DKPBidder:DKP_Sub(msg)
	DKPBidder:MATH_DKP(msg,"sub");
end;

-- Add
function DKPBidder:DKP_Add(msg)
	DKPBidder:MATH_DKP(msg,"add");
end;

-- All in one
function DKPBidder:MATH_DKP(msg,_type)
	_args = GetArgs(msg, " ");
	
	if _type == "add" then
						self.db.global.DkpData.players[_args[1]]["dkp_current"] = self.db.global.DkpData.players[_args[1]]["dkp_current"] + _args[2] ;
						DKPBidder:RaidDisplay("<DKP>".._args[1]..L["win"].._args[2].." dkp");

	elseif _type == "sub" then
						self.db.global.DkpData.players[_args[1]]["dkp_current"] = self.db.global.DkpData.players[_args[1]]["dkp_current"] - _args[2] ;
						DKPBidder:RaidDisplay("<DKP>".._args[1]..L["loose"].._args[2].." dkp");
	end;	
end;





-- --------------
-- Hook with headcount
-- ----------------
function DKPBidder:HeadcountHook(Addon,raidId, lootId, playerName, source, lootCost, lootNote)
 DKPBidder:DKP_Sub(playerName.." "..lootCost)
end



-- --------------
--    Utility functions
-- ------------------

-- Returns the dkp 
function DKPBidder:grabDkp(player)
return self.db.global.DkpData.players[player]["dkp_current"]
end

-- Player exists
function DKPBidder:playerExists(player)
	if (self.db.global.DkpData.players[player]~=nil) then
		return true
	else
		return false
	end
end

-- Function to print in raid channel, with a test if we are in raid !
function DKPBidder:RaidDisplay(text)
	if UnitInRaid("player") then
		SendChatMessage(text, "RAID", this.language);
	else
		print (text)
	end
end

-- Block in whisps
function BlockIncomingChatFrame(msg)
	if (DKPBidder.db.global.Hidewhisps) then
		_args = GetArgs(arg1," ")
		if (_args[1]=="dkp") then
			return true;
		else
			return false;
		end
	end
end

-- Block out whisps
function BlockOutgoingChatFrame(msg)
	if (DKPBidder.db.global.Hidewhisps) then
		_args = GetArgs(arg1," ")
		if (_args[1].." : "==PREFIX) then
			return true;
		else
			return false;
		end
	end
end
